/*
 * TOAD -- A Simple and Powerful C++ GUI Toolkit for the X Window System
 * Copyright (C) 1996-98 by Mark-Andr Hopf
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public 
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA  02111-1307,  USA
 */

/*
 *
 * New++ -- A modified 'new' to discover memory leaks.
 * Copyright (C) 1996-98 by Mark-Andr Hopf
 * Written 1996-98 by Mark-Andr Hopf
 *
 * To use this feature define `SECURE_NEW' in `toad/debug.hh' and recompile
 * the whole library and your programs.
 *
 */

#include <toad/debug.hh>

#ifdef SECURE_NEW
	#undef new
	#include <malloc.h>
	__struct_toad_new_info *__toad_new_info_list=NULL;
	__struct_toad_new_info __toad_new_info;

	// secure memory allocation for toad's 'new'
	//-------------------------------------------
	void* __toad_alloc(size_t size,__struct_toad_new_info* info)
	{
		// reserve memory
		//----------------
		void* ptr = malloc(sizeof(__struct_toad_new_info)+size+sizeof(unsigned long));
		
		// add additional information about allocated memory
		//---------------------------------------------------
		__struct_toad_new_info *node = (__struct_toad_new_info*)ptr;
		node->line = info->line;							// linenumber of 'new'
		node->file = info->file;							// filename of 'new'
		node->size = size;
		node->check1 = 0xCAFECAFE;
		node->check2 = 0xCAFECAFE;
		*((unsigned long*)(ptr+sizeof(__struct_toad_new_info)+size)) = 0xDEADFADE;

		// add reserved memory to the information list
		//---------------------------------------------
		node->next=__toad_new_info_list;
		__toad_new_info_list=node;
	
		// return address
		//----------------
		void *nptr = ptr + sizeof(__struct_toad_new_info);
	
		#ifdef TRACE_MEMORY
		fprintf(file_mem,"allocating memory at file:%s line:%u\n",node->file,node->line);
		fprintf(file_mem,"  adress: 0x%08lx\n",(long)ptr);
		fprintf(file_mem,"  size  : %u\n",size);
		fflush(file_mem);
		#endif

		return nptr;
	}
	
	void __toad_check_free(__struct_toad_new_info *nptr)
	{
		if ( nptr->check1!=0xCAFECAFE || nptr->check2!=0xCAFECAFE )
		{
			printf("boundary check failed during 'delete' (at top)\n");
			return;
		}
		if ( *((unsigned long*)(((void*)nptr)+sizeof(__struct_toad_new_info)+nptr->size)) != 0xDEADFADE )
		{
			printf("boundary check failed during 'delete' (at bottom)\n");
			return;
		}
	}
	
	// free memory, reserved with the function above
	//-----------------------------------------------
	void __toad_free(void *ptr)
	{
//	printf("delete for 0x%08lx\n",(long)ptr);
// printf("  looking for node at 0x%08lx\n",nptr);

		// assume adress was delivered by '__toad_alloc' and calculate the
		// start of the memory information
		//------------------------------------------------------------------
		__struct_toad_new_info *nptr = (__struct_toad_new_info*)(ptr - sizeof(__struct_toad_new_info));
		__struct_toad_new_info *p 	 = __toad_new_info_list;

		#ifdef TRACE_MEMORY
		fprintf(file_mem,"freeing memory allocated at file:%s line:%u\n",nptr->file,nptr->line);
		fprintf(file_mem,"  adress: 0x%08lx\n",(long)nptr);
		fprintf(file_mem,"  size  : %u\n",nptr->size);
		fflush(file_mem);
		#endif


		// search memory information list
		//--------------------------------
		if (p==nptr)
		{
//			printf("  deleting (first) file:%s line:%u (0x%08lx)\n",nptr->file,nptr->line,nptr);
			__toad_new_info_list = __toad_new_info_list->next;
			__toad_check_free(nptr);
			free(nptr);
			return;
		}
		else
		{
			while(p->next!=NULL)
			{
				if (p->next==nptr)
				{
//					printf("  deleting (not first) file:%s line:%u (0x%08lx)\n",nptr->file,nptr->line,nptr);
					p->next=p->next->next;
					__toad_check_free(nptr);
					free(nptr);
					return;
				}
				p=p->next;
			}
		}
		// 'ptr' wasn't part of memory information list
		//----------------------------------------------
		free(ptr);
	}

	void* operator new(size_t size,__struct_toad_new_info* info)
	{
		return __toad_alloc(size,info);
	}

	void* operator new[](size_t size,__struct_toad_new_info* info)
	{
		return __toad_alloc(size,info);
	}

	void operator delete(void *ptr)
	{
		__toad_free(ptr);
	}

	void operator delete[](void *ptr)
	{
		__toad_free(ptr);
	}
	#define new new(__proc_toad_new_info(__FILE__,__LINE__))
#endif
